from bs4 import BeautifulSoup
from captcha_solver import CaptchaSolver
from Commands import Commands
from pathlib import Path
from requests.compat import urljoin
from selenium import webdriver
from selenium.webdriver.chrome.options import Options as ChromeOptions
from selenium.webdriver.common.by import By
from selenium.webdriver.firefox.options import Options as FirefoxOptions
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.safari.options import Options as SafariOptions
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait
from typing import List, Tuple
from webdriver_manager.chrome import ChromeDriverManager
from webdriver_manager.firefox import GeckoDriverManager
import logging


class web_selenium(Commands):
    def __init__(self, SELENIUM_WEB_BROWSER: str = "chrome", **kwargs):
        self.SELENIUM_WEB_BROWSER = SELENIUM_WEB_BROWSER
        self.commands = {"Scrape Text with Selenium": self.scrape_text_with_selenium}

    def scrape_text_with_selenium(self, url: str) -> Tuple[WebDriver, str]:
        logging.getLogger("selenium").setLevel(logging.CRITICAL)

        options_available = {
            "chrome": ChromeOptions,
            "safari": SafariOptions,
            "firefox": FirefoxOptions,
        }

        options = options_available[self.SELENIUM_WEB_BROWSER]()
        options.add_argument(
            "user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.5615.49 Safari/537.36"
        )
        options.add_argument("--headless")
        if self.SELENIUM_WEB_BROWSER == "firefox":
            driver = webdriver.Firefox(
                executable_path=GeckoDriverManager().install(), options=options
            )
        elif self.SELENIUM_WEB_BROWSER == "safari":
            driver = webdriver.Safari(options=options)
        else:
            driver = webdriver.Chrome(
                executable_path=ChromeDriverManager().install(), options=options
            )
        driver.get(url)

        WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.TAG_NAME, "body"))
        )

        try:
            # Check for captcha and solve it
            driver.find_element(By.CSS_SELECTOR, 'img[src^="/captcha/"]')
            captcha_element = driver.find_element(
                By.CSS_SELECTOR, 'img[src^="/captcha/"]'
            )
            if captcha_element:
                captcha_image = captcha_element.get_attribute("src")
                solver = CaptchaSolver("browser")
                captcha_solution = solver.solve_captcha(captcha_image)
                captcha_input = driver.find_element(
                    By.CSS_SELECTOR, 'input[name="captcha"]'
                )
                captcha_input.send_keys(captcha_solution)
                submit_button = driver.find_element(
                    By.CSS_SELECTOR, 'input[type="submit"]'
                )
                submit_button.click()
        except:
            logging.info("No captcha found")
        page_source = driver.execute_script("return document.body.outerHTML;")
        soup = BeautifulSoup(page_source, "html.parser")

        for script in soup(["script", "style"]):
            script.extract()

        text = soup.get_text()
        lines = (line.strip() for line in text.splitlines())
        chunks = (phrase.strip() for line in lines for phrase in line.split("  "))
        text = "\n".join(chunk for chunk in chunks if chunk)
        return text

    @staticmethod
    def scrape_links_with_selenium(driver: WebDriver, url: str) -> List[str]:
        page_source = driver.page_source
        soup = BeautifulSoup(page_source, "html.parser")

        for script in soup(["script", "style"]):
            script.extract()

        hyperlinks = [
            (link.text, urljoin(url, link["href"]))
            for link in soup.find_all("a", href=True)
        ]

        return [f"{link_text} ({link_url})" for link_text, link_url in hyperlinks]

    @staticmethod
    def close_browser(driver: WebDriver) -> None:
        driver.quit()

    @staticmethod
    def add_header(driver: WebDriver) -> None:
        FILE_DIR = Path(__file__).parent.parent
        driver.execute_script(open(f"{FILE_DIR}/js/overlay.js", "r").read())



from Commands import Commands
from Config import Config
from AGiXT import AGiXT


class work_with_ai(Commands):
    def __init__(self, **kwargs):
        agents = Config().get_agents()
        self.commands = {}
        if agents != None:
            for agent in agents:
                if "name" in agent:
                    name = f" AI Agent {agent['name']}"
                    self.commands.update(
                        {f"Ask{name}": self.ask, f"Instruct{name}": self.instruct}
                    )

    def ask(self, prompt: str, agent_name: str = "AGiXT") -> str:
        response = AGiXT(agent_name).run(
            prompt, prompt="chat", websearch=True, websearch_depth=4
        )
        return response

    def instruct(self, prompt: str, agent_name: str = "AGiXT") -> str:
        response = AGiXT(agent_name).run(
            task=prompt, prompt="instruct", websearch=True, websearch_depth=8
        )
        return response



from io import BytesIO
import os.path
import uuid
from base64 import b64decode
import openai
import requests
from PIL import Image
from Commands import Commands
import logging


class image_generator(Commands):
    def __init__(
        self, HUGGINGFACE_API_KEY: str = "", OPENAI_API_KEY: str = "", **kwargs
    ):
        self.HUGGINGFACE_API_KEY = HUGGINGFACE_API_KEY
        self.OPENAI_API_KEY = OPENAI_API_KEY
        if self.HUGGINGFACE_API_KEY or self.OPENAI_API_KEY:
            self.commands = {"Generate Image": self.generate_image}

    def generate_image(self, prompt: str) -> str:
        filename = f"{str(uuid.uuid4())}.jpg"

        if self.OPENAI_API_KEY:
            return self.generate_image_with_dalle(prompt, filename)
        elif self.HUGGINGFACE_API_KEY:
            return self.generate_image_with_hf(prompt, filename)
        else:
            return "No Image Provider Set"

    def generate_image_with_hf(self, prompt: str, filename: str) -> str:
        API_URL = (
            "https://api-inference.huggingface.co/models/CompVis/stable-diffusion-v1-4"
        )
        headers = {"Authorization": f"Bearer {self.HUGGINGFACE_API_TOKEN}"}

        response = requests.post(
            API_URL,
            headers=headers,
            json={
                "inputs": prompt,
            },
        )

        image = Image.open(BytesIO(response.content))
        logging.info(f"Image Generated for prompt:{prompt}")

        image.save(os.path.join(self.WORKING_DIRECTORY, filename))

        return f"Saved to disk:{filename}"

    def generate_image_with_dalle(self, prompt: str, filename: str) -> str:
        openai.api_key = self.OPENAI_API_KEY

        response = openai.Image.create(
            prompt=prompt,
            n=1,
            size="256x256",
            response_format="b64_json",
        )

        logging.info(f"Image Generated for prompt:{prompt}")

        image_data = b64decode(response["data"][0]["b64_json"])

        with open(f"{self.WORKING_DIRECTORY}/{filename}", mode="wb") as png:
            png.write(image_data)

        return f"Saved to disk:{filename}"



from typing import List, Tuple, Union
from urllib.parse import urljoin, urlparse
import requests
from bs4 import BeautifulSoup
from Commands import Commands


class web_requests(Commands):
    def __init__(self, **kwargs):
        self.session = requests.Session()
        self.session.headers.update({"User-Agent": "AGiXT/1.0"})
        self.commands = {
            "Is Valid URL": self.is_valid_url,
            "Sanitize URL": self.sanitize_url,
            "Check Local File Access": self.check_local_file_access,
            "Get Response": self.get_response,
            "Scrape Text": self.scrape_text,
            "Scrape Links": self.scrape_links,
        }

    def is_valid_url(self, url: str) -> bool:
        try:
            result = urlparse(url)
            return all([result.scheme, result.netloc])
        except ValueError:
            return False

    def sanitize_url(self, url: str) -> str:
        return urljoin(url, urlparse(url).path)

    def check_local_file_access(self, url: str) -> bool:
        local_prefixes = [
            "file:///",
            "file://localhost",
            "http://localhost",
            "https://localhost",
        ]
        return any(url.startswith(prefix) for prefix in local_prefixes)

    def get_response(
        self, url: str, timeout: int = 10
    ) -> Union[Tuple[None, str], Tuple[requests.Response, None]]:
        try:
            if self.check_local_file_access(url):
                raise ValueError("Access to local files is restricted")

            if not url.startswith("http://") and not url.startswith("https://"):
                raise ValueError("Invalid URL format")

            sanitized_url = self.sanitize_url(url)

            response = self.session.get(sanitized_url, timeout=timeout)

            if response.status_code >= 400:
                return None, f"Error: HTTP {str(response.status_code)} error"

            return response, None
        except ValueError as ve:
            return None, f"Error: {str(ve)}"

        except requests.exceptions.RequestException as re:
            return None, f"Error: {str(re)}"

    def scrape_text(self, url: str) -> str:
        response, error_message = self.get_response(url)
        if error_message:
            return error_message
        if not response:
            return "Error: Could not get response"

        soup = BeautifulSoup(response.text, "html.parser")

        for script in soup(["script", "style"]):
            script.extract()

        text = soup.get_text()
        lines = (line.strip() for line in text.splitlines())
        chunks = (phrase.strip() for line in lines for phrase in line.split("  "))
        text = "\n".join(chunk for chunk in chunks if chunk)

        return text

    def scrape_links(self, url: str) -> Union[str, List[str]]:
        response, error_message = self.get_response(url)
        if error_message:
            return error_message
        if not response:
            return "Error: Could not get response"
        soup = BeautifulSoup(response.text, "html.parser")

        for script in soup(["script", "style"]):
            script.extract()
        hyperlinks = [
            (link.text, urljoin(url, link["href"]))
            for link in soup.find_all("a", href=True)
        ]

        return [f"{link_text} ({link_url})" for link_text, link_url in hyperlinks]



import os
import os.path
from typing import Generator, List
from Commands import Commands


class file_operations(Commands):
    def __init__(
        self,
        WORKING_DIRECTORY: str = "./WORKSPACE",
        WORKING_DIRECTORY_RESTRICTED: bool = True,
        **kwargs,
    ):
        self.WORKING_DIRECTORY = WORKING_DIRECTORY
        self.WORKING_DIRECTORY_RESTRICTED = WORKING_DIRECTORY_RESTRICTED
        if not os.path.exists(self.WORKING_DIRECTORY):
            os.makedirs(self.WORKING_DIRECTORY)
        self.commands = {
            "Read File": self.read_file,
            "Write to File": self.write_to_file,
            "Append to File": self.append_to_file,
            "Delete File": self.delete_file,
            "Search Files": self.search_files,
        }

    def safe_join(self, base: str, paths) -> str:
        if "/path/to/" in paths:
            paths = paths.replace("/path/to/", "")
        if str(self.WORKING_DIRECTORY_RESTRICTED).lower() == "true":
            new_path = os.path.normpath(os.path.join(base, *paths.split("/")))
            if not os.path.exists(new_path):
                if "." not in new_path:
                    os.makedirs(new_path)
        else:
            new_path = os.path.normpath(os.path.join("/", *paths))
            if not os.path.exists(new_path):
                os.makedirs(new_path)
        return new_path

    @staticmethod
    def split_file(
        content: str, max_length: int = 4000, overlap: int = 0
    ) -> Generator[str, None, None]:
        start = 0
        content_length = len(content)

        while start < content_length:
            end = start + max_length
            if end + overlap < content_length:
                chunk = content[start : end + overlap]
            else:
                chunk = content[start:content_length]
            yield chunk
            start += max_length - overlap

    def read_file(self, filename: str) -> str:
        try:
            filepath = file_operations.safe_join(self.WORKING_DIRECTORY, filename)
            with open(filepath, "r", encoding="utf-8") as f:
                content = f.read()
            return content
        except Exception as e:
            return f"Error: {str(e)}"

    def write_to_file(self, filename: str, text: str) -> str:
        try:
            filepath = file_operations.safe_join(self.WORKING_DIRECTORY, filename)
            directory = os.path.dirname(filepath)
            if not os.path.exists(directory):
                os.makedirs(directory)
            with open(filepath, "w", encoding="utf-8") as f:
                f.write(text)
            return "File written to successfully."
        except Exception as e:
            return f"Error: {str(e)}"

    def append_to_file(self, filename: str, text: str) -> str:
        try:
            filepath = file_operations.safe_join(self.WORKING_DIRECTORY, filename)
            with open(filepath, "a") as f:
                f.write(text)
            return "Text appended successfully."
        except Exception as e:
            return f"Error: {str(e)}"

    def delete_file(self, filename: str) -> str:
        try:
            filepath = file_operations.safe_join(self.WORKING_DIRECTORY, filename)
            os.remove(filepath)
            return "File deleted successfully."
        except Exception as e:
            return f"Error: {str(e)}"

    def search_files(self, directory: str) -> List[str]:
        found_files = []

        if directory in {"", "/"}:
            search_directory = self.WORKING_DIRECTORY
        else:
            search_directory = file_operations.safe_join(
                self.WORKING_DIRECTORY, directory
            )

        for root, _, files in os.walk(search_directory):
            for file in files:
                if file.startswith("."):
                    continue
                relative_path = os.path.relpath(
                    os.path.join(root, file), self.WORKING_DIRECTORY
                )
                found_files.append(relative_path)

        return found_files



from typing import Union, List
import json
from duckduckgo_search import DDGS
from Commands import Commands
import logging

ddgs = DDGS()


class google(Commands):
    def __init__(self, GOOGLE_API_KEY: str = "", **kwargs):
        self.GOOGLE_API_KEY = GOOGLE_API_KEY
        self.commands = {
            "Google Search": self.google_search,
        }
        if self.GOOGLE_API_KEY:
            self.commands["Google Official Search"] = self.google_official_search

    @staticmethod
    def google_search(query: str, num_results: int = 8) -> str:
        search_results = []
        if not query:
            return json.dumps(search_results)
        try:
            results = ddgs.text(query)
            if len(results) > num_results:
                results = results[:num_results]
        except:
            logging.info(
                "Duck Duck Go Search module broke. You may need to try to do `pip install duckduckgo_search --upgrade` to fix this."
            )
            results = None
        if not results:
            return json.dumps(search_results)
        for j in results:
            search_results.append(j)
        return json.dumps(search_results, ensure_ascii=False, indent=4)

    def google_official_search(
        self, query: str, num_results: int = 8
    ) -> Union[str, List[str]]:
        from googleapiclient.discovery import build
        from googleapiclient.errors import HttpError

        try:
            # Get the Google API key and Custom Search Engine ID from the config file
            api_key = self.GOOGLE_API_KEY
            custom_search_engine_id = self.custom_search_engine_id

            # Initialize the Custom Search API service
            service = build("customsearch", "v1", developerKey=api_key)

            # Send the search query and retrieve the results
            result = (
                service.cse()
                .list(q=query, cx=custom_search_engine_id, num=num_results)
                .execute()
            )

            # Extract the search result items from the response
            search_results = result.get("items", [])

            # Create a list of only the URLs from the search results
            search_results_links = [item["link"] for item in search_results]

        except HttpError as e:
            # Handle errors in the API call
            error_details = json.loads(e.content.decode())

            # Check if the error is related to an invalid or missing API key
            if error_details.get("error", {}).get(
                "code"
            ) == 403 and "invalid API key" in error_details.get("error", {}).get(
                "message", ""
            ):
                return "Error: The provided Google API key is invalid or missing."
            else:
                return f"Error: {e}"

        # Return the list of search result URLs
        return search_results_links



import streamlit as st
import bcrypt
from auth_libs.Users import load_users, save_user_data
from auth_libs.Cfig import Cfig
from auth_libs.Users import check_auth_status
from components.agent_selector import agent_selector

check_auth_status()
agent_name, agent = agent_selector()
CFG = Cfig()

# Check if the user is logged in


def logout_button():
    """
    Renders the logout button.
    """
    if st.button("Logout"):
        # Clear session state and redirect to the login page
        st.session_state.clear()
        st.experimental_rerun()  # Redirect to the login page


# Toggle public registrations
def toggle_value(val, to_toggle="allow_reg"):
    setup_cfg = CFG.load_cfg_data()
    for config in setup_cfg["config"]:
        config[to_toggle] = val
    CFG.save_cfg_data(setup_cfg)

    if val == False:
        st.success("Setting Disabled!")
    else:
        st.success("Setting Enabled!")

    st.session_state["regCheck"] = val


# Change password form
def change_password_form():
    st.write("Change Password")
    current_password = st.text_input("Current Password", type="password")
    new_password = st.text_input("New Password", type="password")
    confirm_password = st.text_input("Confirm New Password", type="password")
    if st.button("Change Password"):
        user_data = load_users()
        for user in user_data["users"]:
            if user["email"] == st.session_state["email"]:
                if bcrypt.checkpw(
                    current_password.encode("utf-8"),
                    user["password_hash"].encode("utf-8"),
                ):
                    if new_password == confirm_password:
                        hashed_password = bcrypt.hashpw(
                            new_password.encode("utf-8"), bcrypt.gensalt()
                        )
                        user["password_hash"] = hashed_password.decode("utf-8")
                        save_user_data(user_data)
                        st.success("Password changed successfully!")
                    else:
                        st.error("New password and confirm password do not match.")
                else:
                    st.error("Incorrect current password.")
                break


# Create user form (admin only)
def create_user_form():
    admin_email = CFG.get_admin_email()
    if st.session_state["email"] != admin_email:
        st.error("Only the admin can create new users.")
        return
    st.write("Create User")
    email = st.text_input("Email")
    password = st.text_input("Password", type="password")
    confirm_password = st.text_input("Confirm Password", type="password")
    if st.button("Create User"):
        if not email:
            st.error("Email is required.")
        elif password != confirm_password:
            st.error("Password and confirm password do not match.")
        else:
            user_data = load_users()
            for user in user_data["users"]:
                if user["email"] == email:
                    st.error("User with the same email already exists.")
                    return

            hashed_password = bcrypt.hashpw(password.encode("utf-8"), bcrypt.gensalt())
            new_user = {
                "email": email,
                "password_hash": hashed_password.decode("utf-8"),
                "admin": "False",
            }
            user_data["users"].append(new_user)
            save_user_data(user_data)
            st.success("User created successfully!")


try:
    admin_email = CFG.get_admin_email()
    logout_button()
    if st.session_state["email"] == admin_email and CFG.auth != "SUL":
        setup_cfg = CFG.load_cfg_data()
        for config in setup_cfg["config"]:
            allow_reg = config["allow_reg"]
            allow_uac = config["allow_uac"]
            allow_ucp = config["allow_ucp"]
            allow_uaa = config["allow_uaa"]

        st.session_state["regCheck"] = allow_reg

        if allow_reg == True:
            ar_opposite = False
        else:
            ar_opposite = True

        if allow_uac == True:
            uac_opposite = False
        else:
            uac_opposite = True

        if allow_ucp == True:
            ucp_opposite = False
        else:
            ucp_opposite = True

        if allow_uaa == True:
            uaa_opposite = False
        else:
            uaa_opposite = True

        col1, col2 = st.columns(2)

        with col1:
            if CFG.auth == "MPBR":
                st.checkbox(
                    "Enable Public Registration",
                    value=allow_reg,
                    on_change=toggle_value,
                    args=(ar_opposite, "allow_reg"),
                )
            else:
                st.checkbox(
                    "Enable Public Registration",
                    value=allow_reg,
                    on_change=toggle_value,
                    args=(ar_opposite, "allow_reg"),
                    disabled=True,
                )
            st.checkbox(
                "Enable Users to Create Public Agents",
                value=allow_uac,
                on_change=toggle_value,
                args=(uac_opposite, "allow_uac"),
            )

        with col2:
            st.checkbox(
                "Enable Users To Create Private Agents",
                value=allow_ucp,
                on_change=toggle_value,
                args=(ucp_opposite, "allow_ucp"),
            )
            st.checkbox(
                "Enable Users To Use Admin's Agents",
                value=allow_uaa,
                on_change=toggle_value,
                args=(uaa_opposite, "allow_uaa"),
            )

    change_password_form()
    if st.session_state["email"] == admin_email and CFG.auth != "SUL":
        create_user_form()
except:
    st.write("You are not logged in, or login is not enabled.")



import json
from typing import List, Optional
from Commands import Commands
from AGiXT import AGiXT


class cicd(Commands):
    def __init__(self, **kwargs):
        self.commands = {
            "Evaluate Code": self.evaluate_code,
            "Analyze Pull Request": self.analyze_pull_request,
            "Perform Automated Testing": self.perform_automated_testing,
            "Run CI-CD Pipeline": self.run_ci_cd_pipeline,
            "Improve Code": self.improve_code,
            "Write Tests": self.write_tests,
        }

    def evaluate_code(self, code: str, agent_name: str = "AGiXT") -> List[str]:
        args = [code]
        function_string = "def analyze_code(code: str) -> List[str]:"
        description_string = "Analyzes the given code and returns a list of suggestions for improvements."
        prompt = f"You are now the following python function: ```# {description_string}\n{function_string}```\n\nOnly respond with your `return` value. Args: {args}"
        return AGiXT(agent_name).run(prompt)

    def analyze_pull_request(self, pr_url: str, agent_name: str = "AGiXT") -> List[str]:
        args = [pr_url]
        function_string = "def analyze_pr(pr_url: str) -> List[str]:"
        description_string = "Analyzes the given pull request and returns a list of suggestions for improvements."
        prompt = f"You are now the following python function: ```# {description_string}\n{function_string}```\n\nOnly respond with your `return` value. Args: {args}"
        return AGiXT(agent_name).run(prompt)

    def perform_automated_testing(
        self, test_url: str, agent_name: str = "AGiXT"
    ) -> List[str]:
        args = [test_url]
        function_string = "def perform_testing(test_url: str) -> List[str]:"
        description_string = "Performs automated testing using AI-driven tools and returns a list of test results."
        prompt = f"You are now the following python function: ```# {description_string}\n{function_string}```\n\nOnly respond with your `return` value. Args: {args}"
        return AGiXT(agent_name).run(prompt)

    def improve_code(
        self, suggestions: List[str], code: str, agent_name: str = "AGiXT"
    ) -> str:
        args = [json.dumps(suggestions), code]
        function_string = (
            "def generate_improved_code(suggestions: List[str], code: str) -> str:"
        )
        description_string = "Improves the provided code based on the suggestions provided, making no other changes."
        prompt = f"You are now the following python function: ```# {description_string}\n{function_string}```\n\nOnly respond with your `return` value. Args: {args}"
        return AGiXT(agent_name).run(prompt)

    def write_tests(
        self,
        code: str,
        focus: Optional[List[str]] = None,
        agent_name: str = "AGiXT",
    ) -> str:
        args = [code, json.dumps(focus) if focus else "None"]
        function_string = "def create_test_cases(code: str, focus: Optional[List[str]] = None) -> str:"
        description_string = "Generates test cases for the existing code, focusing on specific areas if required."
        prompt = f"You are now the following python function: ```# {description_string}\n{function_string}```\n\nOnly respond with your `return` value. Args: {args}"
        return AGiXT(agent_name).run(prompt)

    def run_ci_cd_pipeline(self, repo_url: str, agent_name: str = "AGiXT") -> str:
        args = [repo_url]
        function_string = "def run_pipeline(repo_url: str) -> str:"
        description_string = (
            "Runs the entire CI/CD pipeline for the given repository URL."
        )
        prompt = f"You are now the following python function: ```# {description_string}\n{function_string}```\n\nOnly respond with your `return` value. Args: {args}"
        return AGiXT(agent_name).run(prompt)



from Commands import Commands
import os
from threading import Semaphore
import requests
from playsound import playsound
import gtts


class voice(Commands):
    def __init__(
        self,
        ELEVENLABS_API_KEY: str = "",
        USE_MAC_OS_TTS: bool = False,
        USE_BRIAN_TTS: bool = True,
        ELEVENLABS_VOICE: str = "Josh",
        **kwargs,
    ):
        self.ELEVENLABS_API_KEY = ELEVENLABS_API_KEY
        self.USE_MAC_OS_TTS = USE_MAC_OS_TTS
        self.USE_BRIAN_TTS = USE_BRIAN_TTS
        self.ELEVENLABS_VOICE = ELEVENLABS_VOICE
        self.commands = {"Speak with TTS": self.speak}
        self._mutex = Semaphore(1)

    def speak(self, text: str, engine: str = "gtts") -> bool:
        with self._mutex:
            if engine == "elevenlabs" and self.ELEVENLABS_API_KEY:
                return self._elevenlabs_speech(text, self.ELEVENLABS_VOICE)
            elif engine == "macos" and self.USE_MAC_OS_TTS == "True":
                return self._macos_speech(text, self.ELEVENLABS_VOICE)
            elif engine == "brian" and self.USE_BRIAN_TTS == "True":
                return self._brian_speech(text)
            else:
                return self._gtts_speech(text)

    def _gtts_speech(self, text: str) -> bool:
        tts = gtts.gTTS(text)
        tts.save("speech.mp3")
        playsound("speech.mp3", True)
        os.remove("speech.mp3")
        return True

    def _brian_speech(self, text: str) -> bool:
        tts_url = (
            f"https://api.streamelements.com/kappa/v2/speech?voice=Brian&text={text}"
        )
        response = requests.get(tts_url)

        if response.status_code == 200:
            with open("speech.mp3", "wb") as f:
                f.write(response.content)
            playsound("speech.mp3")
            os.remove("speech.mp3")
            return True
        else:
            return False

    def _elevenlabs_speech(self, text: str, voice_index: int = 0) -> bool:
        voices = ["ErXwobaYiN019PkySvjV", "EXAVITQu4vr4xnSDxMaL"]
        headers = {
            "Content-Type": "application/json",
            "xi-api-key": self.ELEVENLABS_API_KEY,
        }

        tts_url = f"https://api.elevenlabs.io/v1/text-to-speech/{voices[voice_index]}"
        response = requests.post(tts_url, headers=headers, json={"text": text})

        if response.status_code == 200:
            with open("speech.mpeg", "wb") as f:
                f.write(response.content)
            playsound("speech.mpeg", True)
            os.remove("speech.mpeg")
            return True
        else:
            return False

    def _macos_speech(self, text: str, voice_index: int = 0) -> bool:
        if voice_index == 0:
            os.system(f'say "{text}"')
        elif voice_index == 1:
            os.system(f'say -v "Ava (Premium)" "{text}"')
        else:
            os.system(f'say -v Samantha "{text}"')
        return True



from Commands import Commands
from Chain import Chain


class chain_commands(Commands):
    def __init__(self, **kwargs):
        self.chains = Chain().get_chains()
        self.commands = {}
        if self.chains != None:
            for chain in self.chains:
                if "name" in chain:
                    self.commands.update(
                        {f"Run Chain: {chain['name']}": self.run_chain}
                    )

    def run_chain(self, chain_name):
        Chain().run_chain(chain_name)
        return "Chain started successfully."



import os
from typing import List
from Commands import Commands
from AGiXT import AGiXT


class create_new_command(Commands):
    def __init__(self, **kwargs):
        self.commands = {"Create a new command": self.create_command}

    def command_exists(self, file_name: str) -> bool:
        return os.path.exists(f"commands/{file_name}.py")

    def create_command(
        self, function_description: str, agent_name: str = "AGiXT"
    ) -> List[str]:
        with open(f"prompts/Create New Command.txt", "r") as f:
            prompt = f.read()
        prompt = prompt.replace("{{NEW_FUNCTION_DESCRIPTION}}", function_description)
        response = AGiXT(agent_name).run(prompt)
        file_name = response.split("class ")[1].split("(")[0]
        code = code.replace("```", "")

        if not self.command_exists(file_name):
            with open(f"commands/{file_name}.py", "w") as f:
                f.write(code)
            return f"Created new command: {file_name}."
        else:
            return f"Command {file_name} already exists. No changes were made."



import json
import random
import requests
from typing import List
from Commands import Commands


class searxng_commands(Commands):
    def __init__(self, SEARXNG_INSTANCE_URL: str = "", **kwargs):
        self.SEARXNG_INSTANCE_URL = SEARXNG_INSTANCE_URL
        self.commands = {"Use The Search Engine": self.search_searx}

    def searx_servers(self):
        try:  # SearXNG - List of these at https://searx.space/
            url = "https://searx.space/data/instances.json"
            response = requests.get(url)
            data = json.loads(response.text)
            search_engines = list(data["instances"].keys())
            return search_engines
        except:
            return ["https://searx.work"]

    def search_searx(self, query: str) -> List[str]:
        if self.SEARXNG_INSTANCE_URL == "":
            searx = self.searx_servers()
            # Pick a random searx server to use since one was not defined.
            random_index = random.randint(0, len(searx) - 1)
            self.SEARXNG_INSTANCE_URL = searx[random_index].rstrip("/")
        else:
            self.SEARXNG_INSTANCE_URL = self.SEARXNG_INSTANCE_URL.rstrip("/")
        payload = {
            "q": query,
            "language": "en",
            "safesearch": 1,
            "format": "json",
        }
        response = requests.get(f"{self.SEARXNG_INSTANCE_URL}/search", params=payload)
        results = response.json()
        summaries = [
            result["title"] + " - " + result["url"] for result in results["results"]
        ]
        return summaries
